@@ -1,24 +1,238 @@
-/* $Id: dom.c 618 2006-11-10 18:18:24Z pajas $ */
-#include <libxml/tree.h>
-#include <libxml/encoding.h>
-#include <libxml/xmlerror.h>
-#include <libxml/xmlmemory.h>
-#include <libxml/parser.h>
-#include <libxml/parserInternals.h>
-#include <libxml/xmlIO.h>
-#include <libxml/xpath.h>
-#include <libxml/xpathInternals.h>
-#include <libxml/globals.h>
-#include <stdio.h>
+/* $Id$
+ *
+ * This is free software, you may use it and distribute it under the same terms as
+ * Perl itself.
+ *
+ * Copyright 2001-2003 AxKit.com Ltd., 2002-2006 Christian Glahn, 2006-2009 Petr Pajas
+*/
+
+#include "dom.h"
+#include "perl-libxml-mm.h"
 
 /* #define warn(string) fprintf(stderr, string) */
 
 #ifdef XS_WARNINGS
-#define xs_warn(string) warn(string) 
+#define xs_warn(string) warn("%s",string) 
 #else
 #define xs_warn(string)
 #endif
 
+void
+domClearPSVIInList(xmlNodePtr list);
+
+void
+domClearPSVI(xmlNodePtr tree) {
+    xmlAttrPtr prop;
+
+    if (tree == NULL)
+        return;
+    if (tree->type == XML_ELEMENT_NODE) {
+        tree->psvi = NULL;
+        prop = tree->properties;
+        while (prop != NULL) {
+            if (tree->type == XML_ATTRIBUTE_NODE)
+                ((xmlAttrPtr) prop)->psvi = NULL;
+            domClearPSVIInList(prop->children);
+            prop = prop->next;
+        }
+    } else if (tree->type == XML_DOCUMENT_NODE) {
+        ((xmlDocPtr) tree)->psvi = NULL;
+    }
+    if (tree->children != NULL)
+        domClearPSVIInList(tree->children);
+}
+
+void
+domClearPSVIInList(xmlNodePtr list) {
+    xmlNodePtr cur;
+
+    if (list == NULL)
+        return;
+    cur = list;
+    while (cur != NULL) {
+        domClearPSVI(cur);
+        cur = cur->next;
+    }
+}
+
+/**
+ * Name: domReconcileNs
+ * Synopsis: void domReconcileNs( xmlNodePtr tree );
+ * @tree: the tree to reconcile
+ *
+ * Reconciles namespacing on a tree by removing declarations
+ * of element and attribute namespaces that are already
+ * declared in the scope of the corresponding node.
+ **/
+
+void
+domAddNsDef(xmlNodePtr tree, xmlNsPtr ns)
+{
+        xmlNsPtr i = tree->nsDef;
+        while(i != NULL && i != ns)
+                i = i->next;
+        if( i == NULL )
+        {
+                ns->next = tree->nsDef;
+                tree->nsDef = ns;
+        }
+}
+
+char
+domRemoveNsDef(xmlNodePtr tree, xmlNsPtr ns)
+{
+        xmlNsPtr i = tree->nsDef;
+
+        if( ns == tree->nsDef )
+        {
+                tree->nsDef = tree->nsDef->next;
+                ns->next = NULL;
+                return(1);
+        }
+        while( i != NULL )
+        {
+                if( i->next == ns )
+                {
+                        i->next = ns->next;
+                        ns->next = NULL;
+                        return(1);
+                }
+                i = i->next;
+        }
+        return(0);
+}
+
+/* ns->next must be NULL, or bad things could happen */
+xmlNsPtr 
+_domAddNsChain(xmlNsPtr c, xmlNsPtr ns)
+{
+        if( c == NULL )
+                return(ns);
+        else
+        {
+                xmlNsPtr i = c;
+                while(i != NULL && i != ns)
+                        i = i->next;
+                if(i == NULL)
+                {
+                        ns->next = c;
+                        return(ns);
+                }
+        }
+        return(c);
+}
+
+/* We need to be smarter with attributes, because the declaration is on the parent element */
+void
+_domReconcileNsAttr(xmlAttrPtr attr, xmlNsPtr * unused)
+{
+        xmlNodePtr tree = attr->parent;
+	if (tree == NULL)
+		return;
+        if( attr->ns != NULL )
+        {
+		xmlNsPtr ns;
+		if ((attr->ns->prefix != NULL) && 
+		    (xmlStrEqual(attr->ns->prefix, BAD_CAST "xml"))) {
+			/* prefix 'xml' has no visible declaration */
+			ns = xmlSearchNsByHref(tree->doc, tree, XML_XML_NAMESPACE);
+			attr->ns = ns;
+			return;
+		} else {
+			ns = xmlSearchNs( tree->doc, tree->parent, attr->ns->prefix );
+		}
+                if( ns != NULL && ns->href != NULL && attr->ns->href != NULL &&
+                    xmlStrcmp(ns->href,attr->ns->href) == 0 )
+                {
+                        /* Remove the declaration from the element */
+                        if( domRemoveNsDef(tree, attr->ns) )
+                                /* Queue up this namespace for freeing */
+                                *unused = _domAddNsChain(*unused, attr->ns);
+
+                        /* Replace the namespace with the one found */
+                        attr->ns = ns;
+                }
+                else
+                {
+                        /* If the declaration is here, we don't need to do anything */
+                        if( domRemoveNsDef(tree, attr->ns) )
+                                domAddNsDef(tree, attr->ns);
+                        else
+                        {
+                                /* Replace/Add the namespace declaration on the element */
+                                attr->ns = xmlCopyNamespace(attr->ns);
+				if (attr->ns) {
+				  domAddNsDef(tree, attr->ns);
+				}
+                        }
+                }
+        }
+}
+
+void
+_domReconcileNs(xmlNodePtr tree, xmlNsPtr * unused)
+{
+        if( tree->ns != NULL )
+        {
+                xmlNsPtr ns = xmlSearchNs( tree->doc, tree->parent, tree->ns->prefix );
+                if( ns != NULL && ns->href != NULL && tree->ns->href != NULL &&
+                    xmlStrcmp(ns->href,tree->ns->href) == 0 )
+                {
+                        /* Remove the declaration (if present) */
+                        if( domRemoveNsDef(tree, tree->ns) )
+                                /* Queue the namespace for freeing */
+                                *unused = _domAddNsChain(*unused, tree->ns);
+                
+                        /* Replace the namespace with the one found */
+                        tree->ns = ns;
+                }
+                else
+                {
+                        /* If the declaration is here, we don't need to do anything */
+                        if( domRemoveNsDef(tree, tree->ns) ) {
+                              domAddNsDef(tree, tree->ns);
+                        }
+                        else
+                        {
+                                /* Restart the namespace at this point */
+                                tree->ns = xmlCopyNamespace(tree->ns);
+                                domAddNsDef(tree, tree->ns);
+                        }
+                }
+        }
+        /* Fix attribute namespacing */
+        if( tree->type == XML_ELEMENT_NODE )
+        {
+                xmlElementPtr ele = (xmlElementPtr) tree;
+                /* attributes is set to xmlAttributePtr, 
+                   but is an xmlAttrPtr??? */
+                xmlAttrPtr attr = (xmlAttrPtr) ele->attributes;
+                while( attr != NULL )
+                {
+                        _domReconcileNsAttr(attr, unused);
+                        attr = attr->next;
+                }
+        }
+        {
+          /* Recurse through all child nodes */
+          xmlNodePtr child = tree->children;
+          while( child != NULL ) 
+          {
+              _domReconcileNs(child, unused);
+              child = child->next;
+            }
+        }
+}
+
+void
+domReconcileNs(xmlNodePtr tree)
+{
+        xmlNsPtr unused = NULL;
+        _domReconcileNs(tree, &unused);
+        if( unused != NULL )
+                xmlFreeNsList(unused);
+}
+
 /**
  * NAME domParseChar
  * TYPE function
@@ -52,19 +266,19 @@ int
 domParseChar( xmlChar *cur, int *len ) 
 {
     unsigned char c;
-	unsigned int val;
+        unsigned int val;
 
-	/*
-	 * We are supposed to handle UTF8, check it's valid
-	 * From rfc2044: encoding of the Unicode values on UTF-8:
-	 *
-	 * UCS-4 range (hex.)           UTF-8 octet sequence (binary)
-	 * 0000 0000-0000 007F   0xxxxxxx
-	 * 0000 0080-0000 07FF   110xxxxx 10xxxxxx
-	 * 0000 0800-0000 FFFF   1110xxxx 10xxxxxx 10xxxxxx 
-	 *
-	 * Check for the 0x110000 limit too
-	 */
+        /*
+         * We are supposed to handle UTF8, check it's valid
+         * From rfc2044: encoding of the Unicode values on UTF-8:
+         *
+         * UCS-4 range (hex.)           UTF-8 octet sequence (binary)
+         * 0000 0000-0000 007F   0xxxxxxx
+         * 0000 0080-0000 07FF   110xxxxx 10xxxxxx
+         * 0000 0800-0000 FFFF   1110xxxx 10xxxxxx 10xxxxxx 
+         *
+         * Check for the 0x110000 limit too
+         */
     
     if ( cur == NULL || *cur == 0 ) {
         *len = 0;
@@ -88,21 +302,21 @@ domParseChar( xmlChar *cur, int *len )
                 val |= (cur[1] & 0x3f) << 6;
                 val |= cur[2] & 0x3f;
             }
-	    } else {
+            } else {
             /* 2-byte code */
             *len = 2;
             val = (cur[0] & 0x1f) << 6;
             val |= cur[1] & 0x3f;
-	    }
+            }
         if ( !IS_CHAR(val) ) {
             *len = -1;
             return(0);
         }
-	    return(val);
+            return(val);
     }
     else {
         /* 1-byte code */
-	    *len = 1;
+            *len = 1;
         return((int)c); 
     }
 }
@@ -115,7 +329,7 @@ domParseChar( xmlChar *cur, int *len )
  *
  * this function is pretty neat, since you can read in well balanced 
  * strings and get a list of nodes, which can be added to any other node.
- * (shure - this should return a doucment_fragment, but still it doesn't)
+ * (sure - this should return a doucment_fragment, but still it doesn't)
  *
  * the code is pretty heavy i think, but deep in my heard i believe it's 
  * worth it :) (e.g. if you like to read a chunk of well-balanced code 
@@ -166,7 +380,7 @@ domReadWellBalancedString( xmlDocPtr doc, xmlChar* block, int repair ) {
 /** 
  * internal helper: insert node to nodelist
  * synopsis: xmlNodePtr insert_node_to_nodelist( leader, insertnode, followup );
- * while leader and followup are allready list nodes. both may be NULL
+ * while leader and followup are already list nodes. both may be NULL
  * if leader is null the parents children will be reset
  * if followup is null the parent last will be reset.
  * leader and followup has to be followups in the nodelist!!!
@@ -208,15 +422,15 @@ domAddNodeToList(xmlNodePtr cur, xmlNodePtr leader, xmlNodePtr followup)
            cur->parent = p;
        }
        
-       if (c1 && 2 && c1!=leader) {
+       if (c1 && c2 && c1!=leader) {
            if ( leader ) {
                leader->next = c1;
-	       c1->prev = leader;
+               c1->prev = leader;
            }
            else if ( p ) {
                p->children = c1;
            }
-	   
+           
            if ( followup ) {
                followup->prev = c2;
                c2->next = followup;
@@ -237,25 +451,25 @@ domAddNodeToList(xmlNodePtr cur, xmlNodePtr leader, xmlNodePtr followup)
  * function returns 1 (TRUE), otherwise 0 (FALSE).
  **/
 int
-domIsParent( xmlNodePtr cur, xmlNodePtr ref ) {
+domIsParent( xmlNodePtr cur, xmlNodePtr refNode ) {
     xmlNodePtr helper = NULL;
 
-    if ( cur == NULL
-         || ref == NULL
-         || cur->doc != ref->doc
-         || ref->children == NULL
+    if ( cur == NULL || refNode == NULL) return 0;
+    if (refNode==cur) return 1;
+    if ( cur->doc != refNode->doc
+         || refNode->children == NULL
          || cur->parent == (xmlNodePtr)cur->doc
          || cur->parent == NULL ) {
         return 0;
     }
 
-    if( ref->type == XML_DOCUMENT_NODE ) {
+    if( refNode->type == XML_DOCUMENT_NODE ) {
         return 1;
     }
 
     helper= cur;
     while ( helper && (xmlDocPtr) helper != cur->doc ) {
-        if( helper == ref ) {
+        if( helper == refNode ) {
             return 1;
         }
         helper = helper->parent;
@@ -265,13 +479,24 @@ domIsParent( xmlNodePtr cur, xmlNodePtr ref ) {
 }
 
 int
-domTestHierarchy(xmlNodePtr cur, xmlNodePtr ref) 
+domTestHierarchy(xmlNodePtr cur, xmlNodePtr refNode) 
 {
-    if ( !ref || !cur || cur->type == XML_ATTRIBUTE_NODE ) {
+    if ( !refNode || !cur ) {
         return 0;
     }
+    if (cur->type == XML_ATTRIBUTE_NODE) {
+      switch ( refNode->type ){
+      case XML_TEXT_NODE:
+      case XML_ENTITY_REF_NODE:
+        return 1;
+        break;
+      default:
+        return 0;
+        break;
+      }
+    }
 
-    switch ( ref->type ){
+    switch ( refNode->type ){
     case XML_ATTRIBUTE_NODE:
     case XML_DOCUMENT_NODE:
         return 0;
@@ -280,7 +505,7 @@ domTestHierarchy(xmlNodePtr cur, xmlNodePtr ref)
         break;
     }
     
-    if ( domIsParent( cur, ref ) ) {
+    if ( domIsParent( cur, refNode ) ) {
         return 0;
     }
 
@@ -288,10 +513,10 @@ domTestHierarchy(xmlNodePtr cur, xmlNodePtr ref)
 }
 
 int
-domTestDocument(xmlNodePtr cur, xmlNodePtr ref)
+domTestDocument(xmlNodePtr cur, xmlNodePtr refNode)
 {
     if ( cur->type == XML_DOCUMENT_NODE ) {
-        switch ( ref->type ) {
+        switch ( refNode->type ) {
         case XML_ATTRIBUTE_NODE:
         case XML_ELEMENT_NODE:
         case XML_ENTITY_NODE:
@@ -341,7 +566,7 @@ domUnlinkNode( xmlNodePtr node ) {
 }
 
 xmlNodePtr
-domImportNode( xmlDocPtr doc, xmlNodePtr node, int move ) {
+domImportNode( xmlDocPtr doc, xmlNodePtr node, int move, int reconcileNS ) {
     xmlNodePtr return_node = node;
 
     if ( move ) {
@@ -355,20 +580,22 @@ domImportNode( xmlDocPtr doc, xmlNodePtr node, int move ) {
             return_node = (xmlNodePtr) xmlCopyDtd((xmlDtdPtr) node);
         }
         else {
-            return_node = xmlCopyNode( node, 1 );
+            return_node = xmlDocCopyNode( node, doc, 1 );
         }
     }
 
 
     /* tell all children about the new boss */ 
-    if ( node && doc && node->doc != doc ) {
+    if ( node && node->doc != doc ) {
+        /* if the source document contained psvi, mark the current document as psvi tainted */
+        if (PmmIsPSVITainted(node->doc))
+            PmmInvalidatePSVI(doc);
         xmlSetTreeDoc(return_node, doc);
     }
 
-    if ( doc != NULL 
-         && return_node != NULL
+    if ( reconcileNS && doc && return_node 
          && return_node->type != XML_ENTITY_REF_NODE ) {
-        xmlReconciliateNs(doc, return_node);     
+                domReconcileNs(return_node);
     }
 
     return return_node;
@@ -477,14 +704,14 @@ domName(xmlNodePtr node) {
 xmlNodePtr
 domAppendChild( xmlNodePtr self,
                 xmlNodePtr newChild ){
+  xmlNodePtr fragment = NULL;
     if ( self == NULL ) {
         return newChild;
     }
 
     if ( !(domTestHierarchy(self, newChild)
            && domTestDocument(self, newChild))){
-        xs_warn("HIERARCHY_REQUEST_ERR\n"); 
-        xmlGenericError(xmlGenericErrorContext,"HIERARCHY_REQUEST_ERR\n");
+        croak("appendChild: HIERARCHY_REQUEST_ERR\n"); 
         return NULL;
     }
 
@@ -494,16 +721,19 @@ domAppendChild( xmlNodePtr self,
     else {
         xs_warn("WRONG_DOCUMENT_ERR - non conform implementation\n"); 
         /* xmlGenericError(xmlGenericErrorContext,"WRONG_DOCUMENT_ERR\n"); */
-        newChild= domImportNode( self->doc, newChild, 1 );
+        newChild = domImportNode( self->doc, newChild, 1, 0 );
     }
  
     if ( self->children != NULL ) {
+        if (newChild->type == XML_DOCUMENT_FRAG_NODE )
+            fragment = newChild->children;
         domAddNodeToList( newChild, self->last, NULL );
     }
     else if (newChild->type == XML_DOCUMENT_FRAG_NODE ) {
         xmlNodePtr c1 = NULL;
         self->children = newChild->children;
-        c1 = newChild->children;
+        fragment = newChild->children;
+        c1 = fragment;
         while ( c1 ){
             c1->parent = self;
             c1 = c1->next;
@@ -517,8 +747,16 @@ domAppendChild( xmlNodePtr self,
         newChild->parent= self;
     }
  
-    if ( newChild->type != XML_ENTITY_REF_NODE ) {
-        xmlReconciliateNs(self->doc, newChild);     
+    if ( fragment ) {
+        /* we must reconcile all nodes in the fragment */
+        newChild = fragment; /* return the first node in the fragment */
+        while ( fragment ) {
+            domReconcileNs(fragment);
+            fragment = fragment->next;
+        }
+    }
+    else if ( newChild->type != XML_ENTITY_REF_NODE ) {
+                domReconcileNs(newChild);
     }
 
     return newChild;
@@ -539,11 +777,17 @@ domRemoveChild( xmlNodePtr self, xmlNodePtr old ) {
     }
 
     domUnlinkNode( old );    
+        if ( old->type == XML_ELEMENT_NODE ) {
+                domReconcileNs( old );
+        }
+
     return old ;
 }
 
 xmlNodePtr
 domReplaceChild( xmlNodePtr self, xmlNodePtr new, xmlNodePtr old ) {
+    xmlNodePtr fragment = NULL;
+    xmlNodePtr fragment_next = NULL;    
     if ( self== NULL )
         return NULL;
 
@@ -562,8 +806,7 @@ domReplaceChild( xmlNodePtr self, xmlNodePtr new, xmlNodePtr old ) {
 
     if ( !(domTestHierarchy(self, new)
            && domTestDocument(self, new))){
-        xs_warn("HIERARCHY_REQUEST_ERR\n"); 
-        xmlGenericError(xmlGenericErrorContext,"HIERARCHY_REQUEST_ERR\n");
+        croak("replaceChild: HIERARCHY_REQUEST_ERR\n");
         return NULL;
     }
     
@@ -572,7 +815,7 @@ domReplaceChild( xmlNodePtr self, xmlNodePtr new, xmlNodePtr old ) {
     }
     else {
         /* WRONG_DOCUMENT_ERR - non conform implementation */
-        new = domImportNode( self->doc, new, 1 );
+        new = domImportNode( self->doc, new, 1, 1 );
     }
     
     if( old == self->children && old == self->last ) {
@@ -582,12 +825,22 @@ domReplaceChild( xmlNodePtr self, xmlNodePtr new, xmlNodePtr old ) {
     else if ( new->type == XML_DOCUMENT_FRAG_NODE 
               && new->children == NULL ) {
         /* want to replace with an empty fragment, then remove ... */
+        fragment = new->children;
+        fragment_next = old->next;
         domRemoveChild( self, old );
     }
     else {
         domAddNodeToList(new, old->prev, old->next );
         old->parent = old->next = old->prev = NULL;    
     }
+    if ( fragment ) {
+        while ( fragment && fragment != fragment_next ) {
+            domReconcileNs(fragment);
+            fragment = fragment->next;
+        }
+    } else if ( new->type != XML_ENTITY_REF_NODE ) {
+                domReconcileNs(new);
+    }
 
     return old;
 }
@@ -597,6 +850,7 @@ xmlNodePtr
 domInsertBefore( xmlNodePtr self, 
                  xmlNodePtr newChild,
                  xmlNodePtr refChild ){
+    xmlNodePtr fragment = NULL;
     if ( refChild == newChild ) {
         return newChild;
     }
@@ -621,7 +875,7 @@ domInsertBefore( xmlNodePtr self,
    
     if ( !(domTestHierarchy( self, newChild )
            && domTestDocument( self, newChild ))) {
-        xmlGenericError(xmlGenericErrorContext,"HIERARCHY_REQUEST_ERR\n");
+        croak("insertBefore/insertAfter: HIERARCHY_REQUEST_ERR\n");
         return NULL;
     }
 
@@ -629,18 +883,27 @@ domInsertBefore( xmlNodePtr self,
         domUnlinkNode( newChild );
     }
     else {
-        newChild = domImportNode( self->doc, newChild, 1 );
+        newChild = domImportNode( self->doc, newChild, 1, 0 );
     }
     
+    if ( newChild->type == XML_DOCUMENT_FRAG_NODE ) {
+      fragment = newChild->children;
+    }
     if ( refChild == NULL ) {
         domAddNodeToList(newChild, self->last, NULL);
     }
     else { 
         domAddNodeToList(newChild, refChild->prev, refChild);
     }
-
-    if ( newChild->type != XML_ENTITY_REF_NODE ) {
-        xmlReconciliateNs(self->doc, newChild);     
+    
+    if ( fragment ) {
+        newChild = fragment; /* return the first node in the fragment */
+        while ( fragment && fragment != refChild ) {
+            domReconcileNs(fragment);
+            fragment = fragment->next;
+        }
+    } else if ( newChild->type != XML_ENTITY_REF_NODE ) {
+                domReconcileNs(newChild);
     }
 
     return newChild;
@@ -661,7 +924,7 @@ domInsertAfter( xmlNodePtr self,
 
 xmlNodePtr
 domReplaceNode( xmlNodePtr oldNode, xmlNodePtr newNode ) {
-    xmlNodePtr prev = NULL, next = NULL, par = NULL;
+    xmlNodePtr prev = NULL, next = NULL, par = NULL, fragment = NULL;
     
     if ( oldNode == NULL
          || newNode == NULL ) {
@@ -677,7 +940,7 @@ domReplaceNode( xmlNodePtr oldNode, xmlNodePtr newNode ) {
          * wrong node type
          * new node is parent of itself
          */
-        xmlGenericError(xmlGenericErrorContext,"HIERARCHY_REQUEST_ERR\n");
+        croak("replaceNode: HIERARCHY_REQUEST_ERR\n");
         return NULL;
     }
         
@@ -692,16 +955,24 @@ domReplaceNode( xmlNodePtr oldNode, xmlNodePtr newNode ) {
         domUnlinkNode( oldNode );
     }
 
+    if ( newNode->type == XML_DOCUMENT_FRAG_NODE ) {
+        fragment = newNode->children;
+    }
     if( prev == NULL && next == NULL ) {
         /* oldNode was the only child */
-        domAppendChild( par ,newNode ); 
+        domAppendChild( par , newNode ); 
     }
     else {
         domAddNodeToList( newNode, prev,  next );
     }
 
-    if ( newNode->type != XML_ENTITY_REF_NODE ) {
-        xmlReconciliateNs(newNode->doc, newNode); 
+    if ( fragment ) {
+        while ( fragment && fragment != next ) {
+            domReconcileNs(fragment);
+            fragment = fragment->next;
+        }
+    } else if ( newNode->type != XML_ENTITY_REF_NODE ) {
+                domReconcileNs(newNode);
     }
 
     return oldNode;
@@ -784,20 +1055,6 @@ domSetNodeValue( xmlNodePtr n , xmlChar* val ){
 }
 
 
-void
-domSetParentNode( xmlNodePtr self, xmlNodePtr p ) {
-    /* never set the parent to a node in the own subtree */ 
-    if( self && !domIsParent(self, p)) {
-        if( self->parent != p ){
-            xmlUnlinkNode( self );
-            self->parent = p;
-            if( p->doc != self->doc ) {
-                self->doc = p->doc;
-            }
-        }
-    }
-}
-
 xmlNodeSetPtr
 domGetElementsByTagName( xmlNodePtr n, xmlChar* name ){
     xmlNodeSetPtr rv = NULL;
@@ -885,15 +1142,15 @@ domGetAttrNode(xmlNodePtr node, const xmlChar *qname) {
     if ( ret == NULL ) {
       localname = xmlSplitQName2(qname, &prefix);
       if ( localname != NULL ) {
-	ns = xmlSearchNs( node->doc, node, prefix );
-	if ( ns != NULL ) {
+        ns = xmlSearchNs( node->doc, node, prefix );
+        if ( ns != NULL ) {
           /* then try localname with the namespace bound to prefix */
-	  ret = xmlHasNsProp( node, localname, ns->href );
-	}
-	if ( prefix != NULL) {
-	  xmlFree( prefix );
-	}
-	xmlFree( localname );
+          ret = xmlHasNsProp( node, localname, ns->href );
+        }
+        if ( prefix != NULL) {
+          xmlFree( prefix );
+        }
+        xmlFree( localname );
       }
     }
     if (ret && ret->type != XML_ATTRIBUTE_NODE) {
@@ -912,10 +1169,10 @@ domSetAttributeNode( xmlNodePtr node, xmlAttrPtr attr ) {
     if ( attr != NULL && attr->type != XML_ATTRIBUTE_NODE )
         return NULL;
     if ( node == attr->parent ) {
-        return attr; /* attribute is allready part of the node */
+        return attr; /* attribute is already part of the node */
     }  
     if ( attr->doc != node->doc ){
-        attr = (xmlAttrPtr) domImportNode( node->doc, (xmlNodePtr) attr, 1 ); 
+        attr = (xmlAttrPtr) domImportNode( node->doc, (xmlNodePtr) attr, 1, 1 ); 
     } 
     else {
         xmlUnlinkNode( (xmlNodePtr) attr );
@@ -946,9 +1203,9 @@ domAttrSerializeContent(xmlBufferPtr buffer, xmlAttrPtr attr)
     while (children != NULL) {
         switch (children->type) {
             case XML_TEXT_NODE:
-	        xmlAttrSerializeTxtContent(buffer, attr->doc,
-		                           attr, children->content);
-		break;
+                xmlAttrSerializeTxtContent(buffer, attr->doc,
+                                           attr, children->content);
+                break;
             case XML_ENTITY_REF_NODE:
                 xmlBufferAdd(buffer, BAD_CAST "&", 1);
                 xmlBufferAdd(buffer, children->name,
@@ -970,9 +1227,6 @@ domNodeNormalize( xmlNodePtr node );
 int
 domNodeNormalizeList( xmlNodePtr nodelist )
 {
-    if ( nodelist == NULL ) 
-        return(0);
-
     while ( nodelist ){
         if ( domNodeNormalize( nodelist ) == 0 )
             return(0);
@@ -998,7 +1252,7 @@ domNodeNormalize( xmlNodePtr node )
             xmlUnlinkNode( next );
 
             /**
-             * keep only nodes that are refered by perl (or GDOME)
+             * keep only nodes that are referred by perl (or GDOME)
              */
             if ( !next->_private )
                 xmlFreeNode( next );
@@ -1007,6 +1261,7 @@ domNodeNormalize( xmlNodePtr node )
     case XML_ELEMENT_NODE:
         domNodeNormalizeList( (xmlNodePtr) node->properties );
     case XML_ATTRIBUTE_NODE:
+    case XML_DOCUMENT_NODE:
         return( domNodeNormalizeList( node->children ) );
         break;
     default:
@@ -1022,42 +1277,43 @@ domRemoveNsRefs(xmlNodePtr tree, xmlNsPtr ns) {
 
     if ((node == NULL) || (node->type != XML_ELEMENT_NODE)) return(0);
     while (node != NULL) {
-	if (node->ns == ns)
-	    node->ns = NULL; /* remove namespace reference */
-	attr = node->properties;
-	while (attr != NULL) {
-	    if (attr->ns == ns)
-		attr->ns = NULL; /* remove namespace reference */
-	    attr = attr->next;
-	}
-	/*
-	 * Browse the full subtree, deep first
-	 */
-	if (node->children != NULL && node->type != XML_ENTITY_REF_NODE) {
-	    /* deep first */
-	    node = node->children;
-	} else if ((node != tree) && (node->next != NULL)) {
-	    /* then siblings */
-	    node = node->next;
-	} else if (node != tree) {
-	    /* go up to parents->next if needed */
-	    while (node != tree) {
-		if (node->parent != NULL)
-		    node = node->parent;
-		if ((node != tree) && (node->next != NULL)) {
-		    node = node->next;
-		    break;
-		}
-		if (node->parent == NULL) {
-		    node = NULL;
-		    break;
-		}
-	    }
-	    /* exit condition */
-	    if (node == tree) 
-		node = NULL;
-	} else
-	    break;
+        if (node->ns == ns)
+            node->ns = NULL; /* remove namespace reference */
+        attr = node->properties;
+        while (attr != NULL) {
+            if (attr->ns == ns)
+                attr->ns = NULL; /* remove namespace reference */
+            attr = attr->next;
+        }
+        /*
+         * Browse the full subtree, deep first
+         */
+        if (node->children != NULL && node->type != XML_ENTITY_REF_NODE) {
+            /* deep first */
+            node = node->children;
+        } else if ((node != tree) && (node->next != NULL)) {
+            /* then siblings */
+            node = node->next;
+        } else if (node != tree) {
+            /* go up to parents->next if needed */
+            while (node != tree) {
+                if (node->parent != NULL)
+                    node = node->parent;
+                if ((node != tree) && (node->next != NULL)) {
+                    node = node->next;
+                    break;
+                }
+                if (node->parent == NULL) {
+                    node = NULL;
+                    break;
+                }
+            }
+            /* exit condition */
+            if (node == tree) 
+                node = NULL;
+        } else
+            break;
     }
     return(1);
 }
+
