@@ -1,3 +1,11 @@
+/* $Id$
+ *
+ * This is free software, you may use it and distribute it under the same terms as
+ * Perl itself.
+ *
+ * Copyright 2001-2003 AxKit.com Ltd., 2002-2006 Christian Glahn, 2006-2009 Petr Pajas
+*/
+
 #include <libxml/tree.h>
 #include <libxml/xpath.h>
 #include <libxml/xpathInternals.h>
@@ -6,6 +14,7 @@
 #include "EXTERN.h"
 
 #include "dom.h"
+#include "xpath.h"
 
 void
 perlDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs){
@@ -141,24 +150,35 @@ perlDocumentFunction(xmlXPathParserContextPtr ctxt, int nargs){
  **/
 
 xmlXPathObjectPtr
-domXPathFind( xmlNodePtr refNode, xmlChar * path ) {
+domXPathFind( xmlNodePtr refNode, xmlChar * path, int to_bool ) {
+    xmlXPathObjectPtr res = NULL;
+    xmlXPathCompExprPtr comp;
+    comp = xmlXPathCompile( path );
+    if ( comp == NULL ) {
+        return NULL;
+    }
+    res = domXPathCompFind(refNode,comp,to_bool);
+    xmlXPathFreeCompExpr(comp);
+    return res;
+}
+
+xmlXPathObjectPtr
+domXPathCompFind( xmlNodePtr refNode, xmlXPathCompExprPtr comp, int to_bool ) {
     xmlXPathObjectPtr res = NULL;
   
-    if ( refNode != NULL && path != NULL ) {
+    if ( refNode != NULL && comp != NULL ) {
         xmlXPathContextPtr ctxt;
-        xmlXPathCompExprPtr comp;
 
         xmlDocPtr tdoc = NULL;
         xmlNodePtr froot = refNode;
 
-        comp = xmlXPathCompile( path );
         if ( comp == NULL ) {
             return NULL;
         }
         
         if ( refNode->doc == NULL ) {
             /* if one XPaths a node from a fragment, libxml2 will
-               refuse the lookup. this is not very usefull for XML
+               refuse the lookup. this is not very useful for XML
                scripters. thus we need to create a temporary document
                to make libxml2 do it's job correctly.
              */
@@ -172,8 +192,9 @@ domXPathFind( xmlNodePtr refNode, xmlChar * path ) {
                 froot = froot->parent;
             }
             xmlAddChild((xmlNodePtr)tdoc, froot);
-
-            refNode->doc = tdoc;
+            xmlSetTreeDoc(froot, tdoc); /* probably no need to clean psvi */
+	    froot->doc = tdoc;
+            /* refNode->doc = tdoc; */
         }
 
         /* prepare the xpath context */
@@ -196,11 +217,21 @@ domXPathFind( xmlNodePtr refNode, xmlChar * path ) {
         xmlXPathRegisterFunc(ctxt,
                              (const xmlChar *) "document",
                              perlDocumentFunction);
-       
-        res = xmlXPathCompiledEval(comp, ctxt);
-
-        xmlXPathFreeCompExpr(comp);
-
+	if (to_bool) {
+#if LIBXML_VERSION >= 20627
+	  int val = xmlXPathCompiledEvalToBoolean(comp, ctxt);
+	  res = xmlXPathNewBoolean(val);
+#else
+	  res = xmlXPathCompiledEval(comp, ctxt);
+	  if (res!=NULL) {
+	    int val = xmlXPathCastToBoolean(res);
+            xmlXPathFreeObject(res);
+	    res = xmlXPathNewBoolean(val);
+	  }
+#endif
+	} else {
+	  res = xmlXPathCompiledEval(comp, ctxt);
+	}
         if (ctxt->namespaces != NULL) {
             xmlFree( ctxt->namespaces );
         }
@@ -210,7 +241,9 @@ domXPathFind( xmlNodePtr refNode, xmlChar * path ) {
         if ( tdoc != NULL ) {
             /* after looking through a fragment, we need to drop the
                fake document again */
-            xmlSetTreeDoc(froot, NULL);
+            xmlSetTreeDoc(froot, NULL); /* probably no need to clean psvi */
+	    froot->doc = NULL;
+	    froot->parent = NULL;
             tdoc->children = NULL;
             tdoc->last     = NULL;
             /* next line is not required anymore */
@@ -222,12 +255,36 @@ domXPathFind( xmlNodePtr refNode, xmlChar * path ) {
     return res;
 }
 
+/* this function is not actually used: */
 xmlNodeSetPtr
 domXPathSelect( xmlNodePtr refNode, xmlChar * path ) {
     xmlNodeSetPtr rv = NULL;
     xmlXPathObjectPtr res = NULL;
   
-    res = domXPathFind( refNode, path );
+    res = domXPathFind( refNode, path, 0 );
+    
+    if (res != NULL) {
+            /* here we have to transfer the result from the internal
+               structure to the return value */
+        	/* get the result from the query */
+        	/* we have to unbind the nodelist, so free object can 
+        	   not kill it */
+        rv = res->nodesetval;  
+        res->nodesetval = 0 ;
+    }
+
+    xmlXPathFreeObject(res);
+
+    return rv;
+}
+
+/* this function is not actually used: */
+xmlNodeSetPtr
+domXPathCompSelect( xmlNodePtr refNode, xmlXPathCompExprPtr comp ) {
+    xmlNodeSetPtr rv = NULL;
+    xmlXPathObjectPtr res = NULL;
+  
+    res = domXPathCompFind( refNode, comp, 0 );
     
     if (res != NULL) {
             /* here we have to transfer the result from the internal
@@ -252,26 +309,34 @@ domXPathSelect( xmlNodePtr refNode, xmlChar * path ) {
  **/
 
 xmlXPathObjectPtr
-domXPathFindCtxt( xmlXPathContextPtr ctxt, xmlChar * path ) {
+domXPathFindCtxt( xmlXPathContextPtr ctxt, xmlChar * path, int to_bool ) {
     xmlXPathObjectPtr res = NULL;
-  
     if ( ctxt->node != NULL && path != NULL ) {
         xmlXPathCompExprPtr comp;
-
-        xmlDocPtr tdoc = NULL;
-        xmlNodePtr froot = ctxt->node;
-
         comp = xmlXPathCompile( path );
         if ( comp == NULL ) {
             return NULL;
         }
-        
+        res = domXPathCompFindCtxt(ctxt,comp,to_bool);
+        xmlXPathFreeCompExpr(comp);
+    }
+    return res;
+}
+
+xmlXPathObjectPtr
+domXPathCompFindCtxt( xmlXPathContextPtr ctxt, xmlXPathCompExprPtr comp, int to_bool ) {
+    xmlXPathObjectPtr res = NULL;
+    if ( ctxt != NULL && ctxt->node != NULL && comp != NULL ) {
+        xmlDocPtr tdoc = NULL;
+        xmlNodePtr froot = ctxt->node;
+
         if ( ctxt->node->doc == NULL ) {
             /* if one XPaths a node from a fragment, libxml2 will
-               refuse the lookup. this is not very usefull for XML
+               refuse the lookup. this is not very useful for XML
                scripters. thus we need to create a temporary document
                to make libxml2 do it's job correctly.
              */
+
             tdoc = xmlNewDoc( NULL );
 
             /* find refnode's root node */
@@ -282,24 +347,36 @@ domXPathFindCtxt( xmlXPathContextPtr ctxt, xmlChar * path ) {
                 froot = froot->parent;
             }
             xmlAddChild((xmlNodePtr)tdoc, froot);
-
-            ctxt->node->doc = tdoc;
+	    xmlSetTreeDoc(froot,tdoc);  /* probably no need to clean psvi */
+            froot->doc = tdoc;
+	    /* ctxt->node->doc = tdoc; */
         }
-       
-        res = xmlXPathCompiledEval(comp, ctxt);
-
-        xmlXPathFreeCompExpr(comp);
-
+	if (to_bool) {
+#if LIBXML_VERSION >= 20627
+	  int val = xmlXPathCompiledEvalToBoolean(comp, ctxt);
+	  res = xmlXPathNewBoolean(val);
+#else
+	  res = xmlXPathCompiledEval(comp, ctxt);
+	  if (res!=NULL) {
+	    int val = xmlXPathCastToBoolean(res);
+            xmlXPathFreeObject(res);
+	    res = xmlXPathNewBoolean(val);
+	  }
+#endif
+	} else {
+	  res = xmlXPathCompiledEval(comp, ctxt);
+	}
         if ( tdoc != NULL ) {
             /* after looking through a fragment, we need to drop the
                fake document again */
-	    xmlSetTreeDoc(froot,NULL);
+	    xmlSetTreeDoc(froot,NULL); /* probably no need to clean psvi */
             froot->doc = NULL;
+            froot->parent  = NULL;
             tdoc->children = NULL;
             tdoc->last     = NULL;
-            froot->parent  = NULL;
-            ctxt->node->doc = NULL;
-
+	    if (ctxt->node) {
+	      ctxt->node->doc = NULL;
+	    }
             xmlFreeDoc( tdoc );
         }
     }
@@ -311,7 +388,7 @@ domXPathSelectCtxt( xmlXPathContextPtr ctxt, xmlChar * path ) {
     xmlNodeSetPtr rv = NULL;
     xmlXPathObjectPtr res = NULL;
   
-    res = domXPathFindCtxt( ctxt, path );
+    res = domXPathFindCtxt( ctxt, path, 0 );
     
     if (res != NULL) {
             /* here we have to transfer the result from the internal
